@@ -232,7 +232,7 @@ enum {
 #define INT_DECODE(interface)					\
 		(interface == WANOPT_RS232) ? "RS232" :	\
 		(interface == WANOPT_V35)	? "V35" :	\
-		"Invalid"
+		(interface == WANOPT_X21)	? "X21" :	"Invalid"
 
 #define SIGNALLING_DECODE(sig)					\
 		(sig == WANOPT_FR_ANSI) ? "ANSI" :	\
@@ -695,10 +695,10 @@ typedef struct wandev_conf
 	unsigned int comm_port;	/* Communication Port (PRI=0, SEC=1) */ 
 	unsigned int bps;	/* data transfer rate */
 	unsigned int mtu;	/* maximum transmit unit size */
-        unsigned udp_port;      /* UDP port for management */
+	unsigned udp_port;      /* UDP port for management */
 	unsigned char ttl;	/* Time To Live for UDP security */
 	unsigned char ft1;	/* FT1 Configurator Option */
-        char electrical_interface;		/* RS-232/V.35, etc. */
+	char electrical_interface;		/* RS-232/V.35, etc. */
 	char clocking;		/* external/internal */
 	char line_coding;	/* NRZ/NRZI/FM0/FM1, etc. */
 	char connection;	/* permanent/switched/on-demand */
