@@ -425,7 +425,7 @@ static int32_t wp_serial_config(void *pfe)
 				/* In this case port is trying to configure for V35
 				 * where previous port already configured for X21 */
 				DEBUG_EVENT("%s: Error: Invalid V35 Configuration, Previous Port configured for X21\n",
-						card->devname,WAN_FE_LINENO(fe));
+						card->devname);
 				return -EINVAL;
 			}
 			wan_clear_bit(AFT_SERIAL_LCFG_X21_MODE_BIT, &reg);
@@ -453,6 +453,8 @@ static int32_t wp_serial_config(void *pfe)
 
 			if (card->wandev.electrical_interface == WANOPT_X21) {
 				aft_serial_write_cpld(card,cpld_reg,0x03);
+			} else {
+				aft_serial_write_cpld(card,cpld_reg,0x01);
 			}
 		}
 		break;
