@@ -18,8 +18,10 @@ cp -rf . $WAN_VIRTUAL/$WZDIR
 
 install -D -m 755 setup-sangoma $WAN_VIRTUAL/usr/local/sbin/setup-sangoma
 install -D -m 755 wancfg_zaptel $WAN_VIRTUAL/usr/sbin/wancfg_zaptel
+install -D -m 755 wancfg_dahdi $WAN_VIRTUAL/usr/sbin/wancfg_dahdi
 install -D -m 755 wancfg_smg $WAN_VIRTUAL/usr/sbin/wancfg_smg
 install -D -m 755 wancfg_tdmapi $WAN_VIRTUAL/usr/sbin/wancfg_tdmapi
+install -D -m 755 wancfg_hp_tdmapi $WAN_VIRTUAL/usr/sbin/wancfg_hp_tdmapi
 
 
 #cp -rf setup-sangoma $WAN_VIRTUAL/usr/local/sbin
